@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,15 +20,15 @@
 #ifndef _CHILLI_H
 #define _CHILLI_H
 
+#include "system.h"
 #include "chilli_limits.h"
 #include "tun.h"
 #include "ippool.h"
 #include "radius.h"
 #include "redir.h"
 #include "syserr.h"
-#include "dhcp.h"
-#include "cmdline.h"
 #include "session.h"
+#include "dhcp.h"
 #include "chilli.h"
 #include "options.h"
 #include "cmdsock.h"
@@ -48,13 +48,17 @@
 #define MPPE_KEYSIZE  16
 #define NT_KEYSIZE    16
 
-
 #define DNPROT_NULL       1
 #define DNPROT_DHCP_NONE  2
 #define DNPROT_UAM        3
 #define DNPROT_WPA        4
+#ifdef ENABLE_EAPOL
 #define DNPROT_EAPOL      5
+#endif
 #define DNPROT_MAC        6
+#ifdef ENABLE_LAYER3
+#define DNPROT_LAYER3     7
+#endif
 
 /* Debug facility */
 #define DEBUG_DHCP        2
@@ -150,11 +154,32 @@ extern struct radius_t *radius;          /* Radius client instance */
 extern struct dhcp_t *dhcp;              /* DHCP instance */
 extern struct tun_t *tun;                /* TUN/TAP instance */
 
+#ifdef ENABLE_CLUSTER
+struct chilli_peer {
+  struct in_addr addr;
+  uint8_t mac[6];
+  uint8_t state;
+  time_t last_update;
+};
+#define PEER_STATE_OFFLINE 0
+#define PEER_STATE_ACTIVE  1
+#define PEER_STATE_STANDBY 2
+#endif
+
 #ifdef ENABLE_STATFILE
 int printstatus();
 int loadstatus();
 #endif
 
+int chilli_connect(struct app_conn_t **appconn, struct dhcp_conn_t *conn);
+
+#ifdef ENABLE_LAYER3
+struct app_conn_t * chilli_connect_layer3(struct in_addr *src, struct dhcp_conn_t *conn);
+#endif
+
+int chilli_getconn(struct app_conn_t **conn, uint32_t ip, 
+		   uint32_t nasip, uint32_t nasport);
+
 int chilli_req_attrs(struct radius_t *radius, 
 		     struct radius_packet_t *pack,
 		     uint32_t service_type,
@@ -178,11 +203,18 @@ int chilli_acct_fromsub(struct app_conn_t *appconn, size_t len);
 int chilli_acct_tosub(struct app_conn_t *appconn, size_t len);
 
 int terminate_appconn(struct app_conn_t *appconn, int terminate_cause);
+
 void config_radius_session(struct session_params *params, 
 			   struct radius_packet_t *pack, 
 			   struct dhcp_conn_t *dhcpconn,
 			   int reconfig);
 
+void session_param_defaults(struct session_params *params);
+
+int dnprot_accept(struct app_conn_t *appconn);
+
+int dnprot_reject(struct app_conn_t *appconn);
+
 int get_urlparts(char *src, char *host, int hostsize, int *port, int *uripos);
 
 int cmdsock_init();
@@ -190,9 +222,18 @@ int cmdsock_init();
 time_t mainclock_tick();
 time_t mainclock_now();
 time_t mainclock_rt();
+time_t mainclock_wall();
+time_t mainclock_towall(time_t t);
 int mainclock_diff(time_t past);
 uint32_t mainclock_diffu(time_t past);
 
+pid_t chilli_fork(uint8_t type, char *name);
+
+#define CHILLI_PROC        0
+#define CHILLI_PROC_DAEMON 1
+#define CHILLI_PROC_REDIR  2
+#define CHILLI_PROC_SCRIPT 3
+
 #ifdef ENABLE_PROXYVSA
 int radius_addvsa(struct radius_packet_t *pack, struct redir_state *state);
 #endif
@@ -220,18 +261,33 @@ void GenerateAuthenticatorResponse(u_char *Password, int PasswordLen,
 				   int UserNameLen, u_char *AuthenticatorResponse);
 #endif
 
+#ifdef ENABLE_MULTIROUTE
+int chilli_getconn_byroute(struct app_conn_t **conn, int idx);
+#endif
+
+int chilli_cmd(struct cmdsock_request *req, bstring s, int sock);
+
 int chilli_handle_signal(void *ctx, int fd);
+void chilli_freeconn();
+
+int runscript(struct app_conn_t *appconn, char* script);
 
 /* utils.c */
 int statedir_file(char *dst, int dlen, char *file, char *deffile);
+int bblk_fromfd(bstring s, int fd, int len);
+int bstring_fromfd(bstring s, int fd);
 
 /* sig.c */
 int ndelay_on (int fd);
 int ndelay_off (int fd);
+int coe (int fd);
+
 int set_signal (int signo, void (*func)(int));
+
 int selfpipe_init (void);
 int selfpipe_read (void);
 int selfpipe_trap (int signo);
+int selfpipe_ignore (int signo);
 void selfpipe_finish();
 
 #endif /*_CHILLI_H */
