@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  */
 
 #include "chilli.h"
+#include "debug.h"
+
+#if defined(ENABLE_CHILLIPROXY) || defined(ENABLE_CHILLIRADSEC) || defined(ENABLE_CHILLIREDIR)
 
 int conn_sock(struct conn_t *conn, struct in_addr *addr, int port) {
   struct sockaddr_in server;
@@ -29,7 +32,9 @@ int conn_sock(struct conn_t *conn, struct in_addr *addr, int port) {
   
   if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) > 0) {
 
+#if(_debug_)
     log_dbg("SETTING non-blocking");
+#endif
     if (ndelay_on(sock) < 0) {
       log_err(errno, "could not set non-blocking");
     }
@@ -96,7 +101,9 @@ void conn_finish(struct conn_t *conn) {
 }
 
 int conn_update_write(struct conn_t *conn) {
+#if(_debug_)
   log_dbg("socket writeable!");
+#endif
   
   if (conn->write_pos == 0) {
     int err;
@@ -106,7 +113,9 @@ int conn_update_write(struct conn_t *conn) {
       conn_finish(conn);
       return -1;
     } else {
+#if(_debug_)
       log_dbg("RESETTING non-blocking");
+#endif
       if (ndelay_off(conn->sock) < 0) {
 	log_err(errno, "could not un-set non-blocking");
       }
@@ -121,7 +130,9 @@ int conn_update_write(struct conn_t *conn) {
       /*log_dbg("write: %d bytes", ret);*/
       conn->write_pos += ret;
     } else if (ret < 0) {
+#if(_debug_)
       log_dbg("socket closed!");
+#endif
       conn_finish(conn);
       return -1;
     }
@@ -140,12 +151,12 @@ int conn_select_update(struct conn_t *conn, select_ctx *sctx) {
 	conn->read_handler(conn, conn->read_handler_ctx);
       }
     }
-
+    
     if (net_select_write_fd(sctx, conn->sock)) {
       conn_update_write(conn);
     }
   }
-
+  
   return 0;
 }
 
@@ -163,7 +174,9 @@ int conn_update(struct conn_t *conn, fd_set *r, fd_set *w, fd_set *e) {
     }
 
     if (FD_ISSET(conn->sock, e)) {
+#if(_debug_)
       log_dbg("socket exception!");
+#endif
       conn_finish(conn);
     }
   }
@@ -182,11 +195,15 @@ _conn_bstring_readhandler(struct conn_t *conn, void *ctx) {
 		  data->mlen - data->slen);
 
   if (ret > 0) {
+#if(_debug_)
     log_dbg("bstring_read: %d bytes", ret);
+#endif
     data->slen += ret;
   } else {
+#if(_debug_)
     log_dbg("socket closed!");
     log_dbg("<== [%s]", data->data);
+#endif
     conn_finish(conn);
   }
 
@@ -220,3 +237,5 @@ int conn_close(struct conn_t *conn) {
   conn->sock = 0;
   return 0;
 }
+
+#endif
