@@ -24,6 +24,7 @@
 #include "pkt.h"
 #include "garden.h"
 #include "net.h"
+#include "bstrlib.h"
 
 /* Option constants */
 #define DHCP_OPTION_MAGIC_LEN       4
@@ -71,6 +72,7 @@
 #define DHCP_BOOTPS 67
 #define DHCP_BOOTPC 68
 #define DHCP_DNS    53
+#define DHCP_MDNS   5353
 
 /* TCP Ports */
 #define DHCP_HTTP   80
@@ -92,6 +94,9 @@ struct dhcp_t; /* Forward declaration */
 #define DHCP_AUTH_AUTH_TOS    4
 #define DHCP_AUTH_DNAT        5
 #define DHCP_AUTH_SPLASH      6
+#ifdef ENABLE_LAYER3
+#define DHCP_AUTH_ROUTER      7
+#endif
 
 #define DHCP_DOMAIN_LEN      30
 
@@ -120,7 +125,7 @@ struct dhcp_conn_t {
   uint8_t noc2c:1;             /* Prevent client to client access using /32 subnets */
   uint8_t is_reserved:1;       /* If this is a static/reserved mapping */
   uint8_t padding:5;
-
+  
   time_t lasttime;             /* Last time we heard anything from client */
   uint8_t hismac[PKT_ETH_ALEN];/* Peer's MAC address */
   struct in_addr ourip;        /* IP address to listen to */
@@ -144,6 +149,7 @@ struct dhcp_conn_t {
   uint16_t tag8021q;
 #endif
 
+#ifdef ENABLE_DHCPRADIUS
   /* XXX: optional */
   struct {
     uint8_t sname[DHCP_SNAME_LEN];     /* 64 Optional server host name, null terminated string.*/
@@ -151,6 +157,7 @@ struct dhcp_conn_t {
     uint8_t options[DHCP_OPTIONS_LEN]; /* var Optional parameters field. */
     size_t option_length;
   } dhcp_opts;
+#endif
 };
 
 
@@ -216,7 +223,9 @@ struct dhcp_t {
   int (*cb_request) (struct dhcp_conn_t *conn, struct in_addr *addr, uint8_t *pack, size_t len);
   int (*cb_connect) (struct dhcp_conn_t *conn);
   int (*cb_disconnect) (struct dhcp_conn_t *conn, int term_cause);
+#ifdef ENABLE_CHILLIQUERY
   int (*cb_getinfo) (struct dhcp_conn_t *conn, bstring b, int fmt);
+#endif
 };
 
 
@@ -267,8 +276,10 @@ int dhcp_set_cb_connect(struct dhcp_t *this,
 int dhcp_set_cb_eap_ind(struct dhcp_t *this, 
   int (*cb_eap_ind) (struct dhcp_conn_t *conn, uint8_t *pack, size_t len));
 
+#ifdef ENABLE_CHILLIQUERY
 int dhcp_set_cb_getinfo(struct dhcp_t *this, 
   int (*cb_getinfo) (struct dhcp_conn_t *conn, bstring b, int fmt));
+#endif
 
 int dhcp_hashget(struct dhcp_t *this, struct dhcp_conn_t **conn, uint8_t *hwaddr);
 
@@ -288,6 +299,11 @@ int dhcp_eapol_ind(struct dhcp_t *this);
 void dhcp_release_mac(struct dhcp_t *this, uint8_t *hwaddr, int term_cause);
 void dhcp_block_mac(struct dhcp_t *this, uint8_t *hwaddr);
 
+void dhcp_authorize_mac(struct dhcp_t *this, uint8_t *hwaddr,
+			struct session_params *params);
+
+void dhcp_reset_tcp_mac(struct dhcp_t *this, uint8_t *hwaddr);
+
 #define LIST_SHORT_FMT 0
 #define LIST_LONG_FMT  1
 #define LIST_JSON_FMT  2
