@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,21 @@
 
 #include "system.h"
 
-int dns_getname(uint8_t **pktp, size_t *left, char *name, size_t namesz, size_t *nameln);
+#define DNS_DEFAULT_MODE 0
+#define DNS_MDNS_MODE 1
 
-char * dns_fullname(char *data, size_t dlen, uint8_t *res, uint8_t *opkt, size_t olen, int lvl);
+int dns_getname(uint8_t **pktp, size_t *left,
+		char *name, size_t namesz, size_t *nameln);
 
-int dns_copy_res(int q, 
+int dns_fullname(char *data, size_t dlen, 
+		 uint8_t *res, size_t reslen,
+		 uint8_t *opkt, size_t olen, 
+		 int lvl);
+
+int dns_copy_res(struct dhcp_conn_t *conn, int q, 
 		 uint8_t **pktp, size_t *left, 
 		 uint8_t *opkt, size_t olen,
-		 char *question, size_t qsize);
+		 uint8_t *question, size_t qsize,
+		 int *qmatch, int mode);
 
 #endif
