@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 #define MAIN_FILE
 
 #include "chilli.h"
+#include "cmdline.h"
+#include "debug.h"
 
 struct options_t _options;
 
@@ -29,7 +31,7 @@ static const char *description =
   "  http://www.coova.org/\n";
 
 static const char *copyright = 
-  "Copyright (c) 2003-2005 Mondru AB., 2006-2010 Coova Technologies LLC, and others.\n"
+  "Copyright (c) 2003-2005 Mondru AB., 2006-2011 Coova Technologies LLC.\n"
   "Licensed under the GNU General Public License (GPL).\n";
 
 static const char *usage =
@@ -39,6 +41,9 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_BINSTATFILE
   "ENABLE_BINSTATFILE "
 #endif
+#ifdef ENABLE_CHILLIQUERY
+  "ENABLE_CHILLIQUERY "
+#endif
 #ifdef ENABLE_CHILLIPROXY
   "ENABLE_CHILLIPROXY "
 #endif
@@ -51,6 +56,24 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_CHILLIXML
   "ENABLE_CHILLIXML "
 #endif
+#ifdef ENABLE_CLUSTER
+  "ENABLE_CLUSTER "
+#endif
+#ifdef ENABLE_DHCPRADIUS
+  "ENABLE_DHCPRADIUS "
+#endif
+#ifdef ENABLE_DHCPOPT
+  "ENABLE_DHCPOPT "
+#endif
+#ifdef ENABLE_DNSLOG
+  "ENABLE_DNSLOG "
+#endif
+#ifdef ENABLE_UAMDOMAINFILE
+  "ENABLE_UAMDOMAINFILE "
+#endif
+#ifdef ENABLE_EAPOL
+  "ENABLE_EAPOL "
+#endif
 #ifdef ENABLE_EWTAPI
   "ENABLE_EWTAPI "
 #endif
@@ -60,18 +83,27 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_IEEE8023
   "ENABLE_IEEE8023 "
 #endif
+#ifdef ENABLE_IPWHITELIST
+  "ENABLE_IPWHITELIST "
+#endif
 #ifdef ENABLE_JSON
   "ENABLE_JSON "
 #endif
 #ifdef ENABLE_LARGELIMITS
   "ENABLE_LARGELIMITS "
 #endif
+#ifdef ENABLE_LAYER3
+  "ENABLE_LAYER3 "
+#endif
 #ifdef ENABLE_LEAKYBUCKET
   "ENABLE_LEAKYBUCKET "
 #endif
 #ifdef ENABLE_MINIPORTAL
   "ENABLE_MINIPORTAL "
 #endif
+#ifdef ENABLE_MULTIROUTE
+  "ENABLE_MULTIROUTE "
+#endif
 #ifdef ENABLE_NETNAT
   "ENABLE_NETNAT "
 #endif
@@ -81,12 +113,15 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_PROXYVSA
   "ENABLE_PROXYVSA "
 #endif
-#ifdef ENABLE_RTMON
-  "ENABLE_RTMON "
+#ifdef ENABLE_REDIRDNSREQ
+  "ENABLE_REDIRDNSREQ "
 #endif
 #ifdef ENABLE_SESSGARDEN
   "ENABLE_SESSGARDEN "
 #endif
+#ifdef ENABLE_SSDP
+  "ENABLE_SSDP "
+#endif
 #ifdef ENABLE_STATFILE
   "ENABLE_STATFILE "
 #endif
@@ -101,6 +136,43 @@ static const char *compile_options = "Compiled with "
 #endif
 ;
 
+char *STRDUP(char *s) {
+  if (!s) return 0;
+  while (isspace(*s)) s++;
+  if (!*s) return 0;
+  return s;
+}
+
+#ifdef ENABLE_MINICONFIG
+
+#define cmdline_parser2 mini_cmdline_parser2
+extern int mini_cmdline_parser2(int argc, char **argv, 
+				struct gengetopt_args_info *args_info, 
+				int, int, int);
+
+#define cmdline_parser_free mini_cmdline_free
+extern int mini_cmdline_free(struct gengetopt_args_info *args_info);
+
+#define cmdline_parser_configfile mini_cmdline_file
+extern int mini_cmdline_file(char *file, struct gengetopt_args_info *args_info,
+			     int, int, int);
+
+static void
+options_print_version (void) {
+  printf ("%s %s\n", PACKAGE, VERSION);
+}
+
+static void
+options_print_help (void) {
+  options_print_version();
+  printf("\n%s", description);
+  printf("\n%s\n", usage);
+  printf("\n%s\n", compile_options);
+  printf("\n%s\n", copyright);
+}
+
+#else
+
 extern const char *gengetopt_args_info_help[];
 
 static void
@@ -111,6 +183,7 @@ options_print_version (void) {
 static void
 options_print_help (void) {
   int i = 0;
+
   options_print_version();
 
   printf("\n%s", description);
@@ -124,12 +197,7 @@ options_print_help (void) {
   printf("\n%s\n", copyright);
 }
 
-char *STRDUP(char *s) {
-  if (!s) return 0;
-  while (isspace(*s)) s++;
-  if (!*s) return 0;
-  return s;
-}
+#endif
 
 int main(int argc, char **argv) {
   struct gengetopt_args_info args_info;
@@ -181,10 +249,19 @@ int main(int argc, char **argv) {
     _options.debug = 0;
 
   /** simple configuration parameters **/
+  _options.layer3 = args_info.layer3_flag;
+#if(_debug_ && !defined(ENABLE_LAYER3))
+  if (_options.layer3) 
+    log_warn(0, "layer3 not implemented. build with --enable-layer3");
+#endif
   _options.uid = args_info.uid_arg;
   _options.gid = args_info.gid_arg;
   _options.mtu = args_info.mtu_arg;
   _options.usetap = args_info.usetap_flag;
+#if(_debug_ && !defined(ENABLE_TAP))
+  if (_options.usetap) 
+    log_warn(0, "tap not implemented. build with --enable-tap");
+#endif
   _options.foreground = args_info.fg_flag;
   _options.interval = args_info.interval_arg;
   _options.lease = args_info.lease_arg;
@@ -192,6 +269,10 @@ int main(int argc, char **argv) {
   _options.dhcpstart = args_info.dhcpstart_arg;
   _options.dhcpend = args_info.dhcpend_arg;
   _options.eapolenable = args_info.eapolenable_flag;
+#if(_debug_ && !defined(ENABLE_EAPOL))
+  if (_options.eapolenable) 
+    log_warn(0, "EAPOL not implemented. build with --enable-eapol");
+#endif
   _options.swapoctets = args_info.swapoctets_flag;
   _options.logfacility = args_info.logfacility_arg;
   _options.chillixml = args_info.chillixml_flag;
@@ -202,6 +283,7 @@ int main(int argc, char **argv) {
   _options.uamuiport = args_info.uamuiport_arg;
   _options.macallowlocal = args_info.macallowlocal_flag;
   _options.strictmacauth = args_info.strictmacauth_flag;
+  _options.strictdhcp = args_info.strictdhcp_flag;
   _options.no_wispr1 = args_info.nowispr1_flag;
   _options.no_wispr2 = args_info.nowispr2_flag;
   _options.wpaguests = args_info.wpaguests_flag;
@@ -222,6 +304,10 @@ int main(int argc, char **argv) {
   _options.radiusretry = args_info.radiusretry_arg;
   _options.radiusretrysec = args_info.radiusretrysec_arg;
   _options.proxyport = args_info.proxyport_arg;
+#if(_debug_ && !defined(ENABLE_RADPROXY))
+  if (_options.proxyport)
+    log_err(0,"radproxy not implemented. build with --enable-radproxy");
+#endif
   _options.txqlen = args_info.txqlen_arg;
   _options.ringsize = args_info.ringsize_arg;
   _options.sndbuf = args_info.sndbuf_arg;
@@ -241,13 +327,31 @@ int main(int argc, char **argv) {
   _options.seskeepalive = args_info.seskeepalive_flag;
   _options.uamallowpost = args_info.uamallowpost_flag;
   _options.redir = args_info.redir_flag;
+#if(_debug_ && !defined(ENABLE_CHILLIREDIR))
+  if (_options.redir) 
+    log_err(0, "chilli_redir not implemented. build with --enable-chilliredir");
+#endif
   _options.redirssl = args_info.redirssl_flag;
   _options.uamuissl = args_info.uamuissl_flag;
   _options.domaindnslocal = args_info.domaindnslocal_flag;
   _options.framedservice = args_info.framedservice_flag;
   _options.radsec = args_info.radsec_flag;
+#if(_debug_ && !defined(ENABLE_CHILLIRADSEC))
+  if (_options.radsec) 
+    log_err(0, "chilli_radsec not implemented. build with --enable-chilliradsec");
+#endif
   _options.proxymacaccept = args_info.proxymacaccept_flag;
+  _options.noradallow = args_info.noradallow_flag;
   _options.peerid = args_info.peerid_arg;
+#if(_debug_ && !defined(ENABLE_CLUSTER))
+  if (_options.peerid) 
+    log_err(0, "clustering not implemented. build with --enable-cluster");
+#endif
+  _options.redirdnsreq = args_info.redirdnsreq_flag;
+#if(_debug_ && !defined(ENABLE_REDIRDNSREQ))
+  if (_options.redirdnsreq) 
+    log_err(0, "redirdnsreq not implemented. build with --enable-redirdnsreq");
+#endif
 
 #ifdef ENABLE_LEAKYBUCKET
   _options.bwbucketupsize = args_info.bwbucketupsize_arg;
@@ -448,7 +552,7 @@ int main(int argc, char **argv) {
     }
   }
 
-  if (args_info.uamhomepage_arg) {
+  if (args_info.uamhomepage_arg && *args_info.uamhomepage_arg) {
     if (get_urlparts(args_info.uamhomepage_arg, hostname, USERURLSIZE, 
 		     &_options.uamhomepageport, 0)) {
       log_err(0,"Failed to parse uamhomepage: %s!", 
@@ -485,6 +589,7 @@ int main(int argc, char **argv) {
   _options.uamnatanyip = args_info.uamnatanyip_flag;
   _options.dnsparanoia = args_info.dnsparanoia_flag;
   _options.radiusoriginalurl = args_info.radiusoriginalurl_flag;
+  _options.routeonetone = args_info.routeonetone_flag;
 
   /* pass-throughs */
   memset(_options.pass_throughs, 0, sizeof(_options.pass_throughs));
@@ -497,8 +602,62 @@ int main(int argc, char **argv) {
 			      args_info.uamallowed_arg[numargs]);
   }
 
+#ifdef ENABLE_DHCPOPT
+  _options.dhcp_options_len = 0;
+  for (numargs = 0; numargs < args_info.dhcpopt_given; ++numargs) {
+    unsigned char binopt[128];
+    int hex_length = strlen(args_info.dhcpopt_arg[numargs]);
+    int bin_length = hex_length / 2;
+    if (hex_length > 0 && (bin_length * 2) == hex_length &&
+	bin_length < sizeof(binopt)) {
+      log_dbg("DHCP Options %s", args_info.dhcpopt_arg[numargs]);
+      if (redir_hextochar((unsigned char *)args_info.dhcpopt_arg[numargs],
+			  hex_length, binopt, bin_length) == 0) {
+	if (_options.dhcp_options_len + bin_length < 
+	    sizeof(_options.dhcp_options)) {
+	  memcpy(_options.dhcp_options + 
+		 _options.dhcp_options_len, 
+		 binopt, bin_length);
+	  _options.dhcp_options_len += bin_length;
+	} else {
+	  log_dbg("No room for DHCP option %d", (int)binopt[0]);
+	}
+      } else {
+	log_dbg("Bad DHCP option hex encoding");
+      }
+    } else {
+      log_dbg("DHCP options are hex encoded binary");
+    }
+  }
+#endif
+
+#ifdef ENABLE_MODULES
+  memset(_options.modules, 0, sizeof(_options.modules));
+  for (numargs = 0; numargs < args_info.module_given; ++numargs) {
+    if (numargs < MAX_MODULES) {
+      char *n, *sc;
+      int len, nlen;
+
+      n = args_info.module_arg[numargs];
+      len = strlen(n);
+      sc = strchr(n, ';');
+      if (!sc) sc = strchr(n, ':');
+      nlen = sc ? (sc - n) : len;
+
+      safe_snprintf(_options.modules[numargs].name, 
+		    sizeof(_options.modules[numargs].name),
+		    "%.*s", nlen, n);
+      if (sc && len > (nlen + 1)) {
+	safe_snprintf(_options.modules[numargs].conf, 
+		      sizeof(_options.modules[numargs].conf),
+		      "%.*s", len - nlen - 1, sc + 1);
+      }
+    }
+  }
+#endif
 
 #ifdef ENABLE_CHILLIREDIR
+  /*
   for (numargs = 0; numargs < MAX_REGEX_PASS_THROUGHS; ++numargs) {
     if (_options.regex_pass_throughs[numargs].re_host.allocated)
       regfree(&_options.regex_pass_throughs[numargs].re_host);
@@ -507,6 +666,7 @@ int main(int argc, char **argv) {
     if (_options.regex_pass_throughs[numargs].re_qs.allocated)
       regfree(&_options.regex_pass_throughs[numargs].re_qs);
   }
+  */
   
   memset(_options.regex_pass_throughs, 0, sizeof(_options.regex_pass_throughs));
   _options.regex_num_pass_throughs = 0;
@@ -701,6 +861,7 @@ int main(int argc, char **argv) {
   /* If no listen option is specified listen to any local port    */
   /* Do hostname lookup to translate hostname to IP address       */
   if (args_info.proxylisten_arg) {
+#ifdef ENABLE_RADPROXY
     if (!(host = gethostbyname(args_info.proxylisten_arg))) {
       log_err(0, "Invalid listening address: %s! [%s]", 
 	      args_info.proxylisten_arg, strerror(errno));
@@ -712,10 +873,14 @@ int main(int argc, char **argv) {
   }
   else {
     _options.proxylisten.s_addr = htonl(INADDR_ANY);
+#elif (_debug_)
+    log_warn(0,"radproxy not implemented. build with --enable-radproxy");
+#endif
   }
   
   /* Store proxyclient as in_addr net and mask                       */
   if (args_info.proxyclient_arg) {
+#ifdef ENABLE_RADPROXY
     if(option_aton(&_options.proxyaddr, &_options.proxymask, 
 		   args_info.proxyclient_arg, 0)) {
       log_err(0,"Invalid proxy client address: %s!", args_info.proxyclient_arg);
@@ -725,6 +890,9 @@ int main(int argc, char **argv) {
   else {
     _options.proxyaddr.s_addr = ~0; /* Let nobody through */
     _options.proxymask.s_addr = 0; 
+#elif (_debug_)
+    log_warn(0,"radproxy not implemented. build with --enable-radproxy");
+#endif
   }
 
   memset(_options.macok, 0, sizeof(_options.macok));
@@ -787,50 +955,22 @@ int main(int argc, char **argv) {
   }
 
   /** string parameters **/
-  if (_options.peerkey) free(_options.peerkey);
-  _options.peerkey = STRDUP(args_info.peerkey_arg);
-
-  if (_options.routeif) free(_options.routeif);
-  _options.routeif = STRDUP(args_info.routeif_arg);
-
-  if (_options.wwwdir) free(_options.wwwdir);
-  _options.wwwdir = STRDUP(args_info.wwwdir_arg);
-
-  if (_options.wwwbin) free(_options.wwwbin);
-  _options.wwwbin = STRDUP(args_info.wwwbin_arg);
-
-  if (_options.uamui) free(_options.uamui);
-  _options.uamui = STRDUP(args_info.uamui_arg);
-
-  if (_options.localusers) free(_options.localusers);
-  _options.localusers = STRDUP(args_info.localusers_arg);
-
 #ifdef HAVE_SSL
-  if (_options.sslkeyfile) free(_options.sslkeyfile);
   _options.sslkeyfile = STRDUP(args_info.sslkeyfile_arg);
-
-  if (_options.sslkeypass) free(_options.sslkeypass);
   _options.sslkeypass = STRDUP(args_info.sslkeypass_arg);
-
-  if (_options.sslcertfile) free(_options.sslcertfile);
   _options.sslcertfile = STRDUP(args_info.sslcertfile_arg);
-
-  if (_options.sslcafile) free(_options.sslcafile);
   _options.sslcafile = STRDUP(args_info.sslcafile_arg);
 #endif
 
 #ifdef USING_IPC_UNIX
-  if (_options.unixipc) free(_options.unixipc);
   _options.unixipc = STRDUP(args_info.unixipc_arg);
 #endif
 
 #ifdef HAVE_NETFILTER_COOVA
-  if (_options.kname) free(_options.kname);
   _options.kname = STRDUP(args_info.kname_arg);
 #endif
 
 #ifdef ENABLE_DNSLOG
-  if (_options.dnslog) free(_options.dnslog);
   _options.dnslog = STRDUP(args_info.dnslog_arg);
 #else
   if (args_info.dnslog_arg)
@@ -838,29 +978,26 @@ int main(int argc, char **argv) {
 #endif
 
 #ifdef ENABLE_IPWHITELIST
-  if (_options.ipwhitelist) free(_options.ipwhitelist);
   _options.ipwhitelist = STRDUP(args_info.ipwhitelist_arg);
 #else
   if (args_info.ipwhitelist_arg)
     log_err(0, "option ipwhitelist given when no support built-in");
 #endif
+
+#ifdef ENABLE_UAMDOMAINFILE
+  _options.uamdomainfile = STRDUP(args_info.uamdomainfile_arg);
+#else
+  if (args_info.uamdomainfile_arg)
+    log_err(0, "option uamdomainfile given when no support built-in");
+#endif
+
+#ifdef ENABLE_MODULES
+  _options.moddir = STRDUP(args_info.moddir_arg);
+#else
+  if (args_info.moddir_arg)
+    log_err(0, "option moddir given when no support built-in");
+#endif
   
-  if (_options.uamurl) free(_options.uamurl);
-  _options.uamurl = STRDUP(args_info.uamserver_arg);
-
-  if (_options.uamaaaurl) free(_options.uamaaaurl);
-  _options.uamaaaurl = STRDUP(args_info.uamaaaurl_arg);
-
-  if (_options.uamhomepage) free(_options.uamhomepage);
-  _options.uamhomepage = STRDUP(args_info.uamhomepage_arg);
-
-  if (_options.wisprlogin) free(_options.wisprlogin);
-  _options.wisprlogin = STRDUP(args_info.wisprlogin_arg);
-
-  if (_options.uamsecret) free(_options.uamsecret);
-  _options.uamsecret = STRDUP(args_info.uamsecret_arg);
-
-  if (_options.proxysecret) free(_options.proxysecret);
   if (!args_info.proxysecret_arg) {
     _options.proxysecret = STRDUP(args_info.radiussecret_arg);
   }
@@ -868,97 +1005,47 @@ int main(int argc, char **argv) {
     _options.proxysecret = STRDUP(args_info.proxysecret_arg);
   }
 
-  if (_options.macsuffix) free(_options.macsuffix);
+  _options.peerkey = STRDUP(args_info.peerkey_arg);
+  _options.routeif = STRDUP(args_info.routeif_arg);
+  _options.wwwdir = STRDUP(args_info.wwwdir_arg);
+  _options.wwwbin = STRDUP(args_info.wwwbin_arg);
+  _options.uamui = STRDUP(args_info.uamui_arg);
+  _options.localusers = STRDUP(args_info.localusers_arg);
+  _options.uamurl = STRDUP(args_info.uamserver_arg);
+  _options.uamaaaurl = STRDUP(args_info.uamaaaurl_arg);
+  _options.uamhomepage = STRDUP(args_info.uamhomepage_arg);
+  _options.wisprlogin = STRDUP(args_info.wisprlogin_arg);
+  _options.uamsecret = STRDUP(args_info.uamsecret_arg);
   _options.macsuffix = STRDUP(args_info.macsuffix_arg);
-
-  if (_options.macpasswd) free(_options.macpasswd);
   _options.macpasswd = STRDUP(args_info.macpasswd_arg);
-
-  if (_options.adminuser) free(_options.adminuser);
   _options.adminuser = STRDUP(args_info.adminuser_arg);
-
-  if (_options.adminpasswd) free(_options.adminpasswd);
   _options.adminpasswd = STRDUP(args_info.adminpasswd_arg);
-
-  if (_options.adminupdatefile) free(_options.adminupdatefile);
   _options.adminupdatefile = STRDUP(args_info.adminupdatefile_arg);
-
-  if (_options.rtmonfile) free(_options.rtmonfile);
   _options.rtmonfile = STRDUP(args_info.rtmonfile_arg);
-
-  if (_options.ssid) free(_options.ssid);
   _options.ssid = STRDUP(args_info.ssid_arg);
-
-  if (_options.vlan) free(_options.vlan);
   _options.vlan = STRDUP(args_info.vlan_arg);
-
-  if (_options.nasmac) free(_options.nasmac);
   _options.nasmac = STRDUP(args_info.nasmac_arg);
-
-  if (_options.nasip) free(_options.nasip);
   _options.nasip = STRDUP(args_info.nasip_arg);
-
-  if (_options.tundev) free(_options.tundev);
   _options.tundev = STRDUP(args_info.tundev_arg);
-
-  if (_options.radiusnasid) free(_options.radiusnasid);
   _options.radiusnasid = STRDUP(args_info.radiusnasid_arg);
-
-  if (_options.radiuslocationid) free(_options.radiuslocationid);
   _options.radiuslocationid = STRDUP(args_info.radiuslocationid_arg);
-
-  if (_options.radiuslocationname) free(_options.radiuslocationname);
   _options.radiuslocationname = STRDUP(args_info.radiuslocationname_arg);
-
-  if (_options.locationname) free(_options.locationname);
   _options.locationname = STRDUP(args_info.locationname_arg);
-
-  if (_options.radiussecret) free(_options.radiussecret);
   _options.radiussecret = STRDUP(args_info.radiussecret_arg);
-
-  if (_options.cmdsocket) free(_options.cmdsocket);
   _options.cmdsocket = STRDUP(args_info.cmdsocket_arg);
-
-  if (_options.domain) free(_options.domain);
   _options.domain = STRDUP(args_info.domain_arg);
-
-  if (_options.ipup) free(_options.ipup);
   _options.ipup = STRDUP(args_info.ipup_arg);
-
-  if (_options.ipdown) free(_options.ipdown);
   _options.ipdown = STRDUP(args_info.ipdown_arg);
-
-  if (_options.conup) free(_options.conup);
   _options.conup = STRDUP(args_info.conup_arg);
-
-  if (_options.condown) free(_options.condown);
   _options.condown = STRDUP(args_info.condown_arg);
-
-  if (_options.macup) free(_options.macup);
   _options.macup = STRDUP(args_info.macup_arg);
-
-  if (_options.macdown) free(_options.macdown);
   _options.macdown = STRDUP(args_info.macdown_arg);
-
-  if (_options.pidfile) free(_options.pidfile);
   _options.pidfile = STRDUP(args_info.pidfile_arg);
-
-  if (_options.statedir) free(_options.statedir);
   _options.statedir = STRDUP(args_info.statedir_arg);
-
-  if (_options.usestatusfile) free(_options.usestatusfile);
   _options.usestatusfile = STRDUP(args_info.usestatusfile_arg);
-
-  if (_options.uamaliasname) free(_options.uamaliasname);
   _options.uamaliasname = STRDUP(args_info.uamaliasname_arg);
-
-  if (_options.uamhostname) free(_options.uamhostname);
   _options.uamhostname = STRDUP(args_info.uamhostname_arg);
-
-  if (_options.binconfig) free(_options.binconfig);
   _options.binconfig = STRDUP(args_info.bin_arg);
-
-  if (_options.ethers) free(_options.ethers);
   _options.ethers = STRDUP(args_info.ethers_arg);
 
   ret = 0;
@@ -983,3 +1070,4 @@ int main(int argc, char **argv) {
 
   return ret;
 }
+
