@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2006 PicoPoint B.V.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,19 +38,29 @@ typedef struct _cmd_info {
 static cmd_info commands[] = {
   { CMDSOCK_LIST,          "list",          NULL },
   { CMDSOCK_LIST_IPPOOL,   "listippool",    NULL },
+  { CMDSOCK_LIST_RADQUEUE, "listradqueue",  NULL },
+  { CMDSOCK_LIST_GARDEN,   "listgarden",    NULL },
   { CMDSOCK_ENTRY_FOR_IP,  "listip",        NULL },
   { CMDSOCK_ENTRY_FOR_MAC, "listmac",       NULL },
-  { CMDSOCK_ROUTE,         "route",         NULL },
-  { CMDSOCK_ROUTE_GW,      "routegw",       NULL },
   { CMDSOCK_RELOAD,        "reload",        NULL },
   { CMDSOCK_DHCP_LIST,     "dhcp-list",     NULL },
   { CMDSOCK_DHCP_RELEASE,  "dhcp-release",  NULL },
   { CMDSOCK_AUTHORIZE,     "authorize",     NULL },
   { CMDSOCK_LOGIN,         "login",         NULL },
-  { CMDSOCK_DHCP_RELEASE,  "logout",        NULL },
-  { CMDSOCK_DHCP_RELEASE,  "logoff",        NULL },
+  { CMDSOCK_UPDATE,        "update",        NULL },
+  { CMDSOCK_LOGOUT,        "logout",        NULL },
+  { CMDSOCK_LOGOUT,        "logoff",        NULL },
   { CMDSOCK_DHCP_DROP,     "drop",          NULL },
   { CMDSOCK_DHCP_DROP,     "block",         NULL },
+  { CMDSOCK_PROCS,         "procs",         NULL },
+#ifdef ENABLE_MULTIROUTE
+  { CMDSOCK_ROUTE,         "route",         NULL },
+  { CMDSOCK_ROUTE_GW,      "routegw",       NULL },
+#endif
+#ifdef ENABLE_CLUSTER
+  { CMDSOCK_PEERS,         "peers",         NULL },
+  { CMDSOCK_PEER_SET,      "peerset",       NULL },
+#endif
 #ifdef ENABLE_STATFILE
   { CMDSOCK_STATUSFILE,    "statusfile",    NULL },
 #endif
@@ -143,6 +153,12 @@ static struct cmd_arguments args[] = {
     &request.data.sess.params.url,
     "Set splash page",
     &request.data.sess.params.flags, REQUIRE_UAM_SPLASH },
+  { "url",
+    CMDSOCK_FIELD_STRING, 
+    sizeof(request.data.sess.params.url),
+    &request.data.sess.params.url,
+    "Set redirect url",
+    &request.data.sess.params.flags, REQUIRE_REDIRECT },
   { "routeidx",
     CMDSOCK_FIELD_INTEGER, 
     sizeof(request.data.sess.params.routeidx),
@@ -212,6 +228,10 @@ int main(int argc, char **argv) {
   int argidx = 1;
 
   struct itimerval itval;
+
+#ifdef ENABLE_CLUSTER
+  int peerid = -1;
+#endif
   
   set_signal(SIGALRM, timeout_alarm);
   
@@ -240,6 +260,12 @@ int main(int argc, char **argv) {
       argidx++;
       if (argidx >= argc) return usage(argv[0]);
       cmdsock = argv[argidx++];
+#ifdef ENABLE_CLUSTER
+    } else if (!strcmp(argv[argidx], "-p")) {
+      argidx++;
+      if (argidx >= argc) return usage(argv[0]);
+      peerid = atoi(argv[argidx++]);
+#endif
     } else if (!strcmp(argv[argidx], "-json")) {
       request.options |= CMDSOCK_OPT_JSON;
       argidx++;
@@ -252,27 +278,29 @@ int main(int argc, char **argv) {
   for (s = 0; commands[s].command; s++) {
     if (!strcmp(cmd, commands[s].command)) {
       request.type = commands[s].type;
+
       switch(request.type) {
+
       case CMDSOCK_LOGIN:
+      case CMDSOCK_LOGOUT:
+      case CMDSOCK_UPDATE:
       case CMDSOCK_AUTHORIZE:
 	{
-	  int pos = argidx;
+	  int c = argc - argidx;
 	  
-	  argc -= argidx;
-	  
-	  while(argc > 0) {
+	  while(c > 0) {
 	    int i;
 	    
 	    for (i=0; i<count; i++) {
 	      
-	      if (!strcmp(argv[pos],args[i].name)) {
+	      if (!strcmp(argv[argidx],args[i].name)) {
 		
 		if (args[i].flag) {
 		  *(args[i].flag) |= args[i].flagbit;
 		}
 		
-		if (argc == 1 && args[i].length) {
-		  fprintf(stderr, "Argument %s requires a value\n", argv[pos]);
+		if (c == 1 && args[i].length) {
+		  fprintf(stderr, "Argument %s requires a value\n", argv[argidx]);
 		  return usage(argv[0]);
 		}
 		
@@ -280,27 +308,27 @@ int main(int argc, char **argv) {
 		case CMDSOCK_FIELD_NONE:
 		  break;
 		case CMDSOCK_FIELD_STRING:
-		  strncpy(((char *)args[i].field), argv[pos+1], args[i].length-1);
+		  safe_strncpy(((char *)args[i].field), argv[argidx+1], args[i].length);
 		  break;
 		case CMDSOCK_FIELD_INTEGER:
 		  switch(args[i].length) {
 		  case 1:
-		    *((uint8_t *)args[i].field) |= (uint8_t)atoi(argv[pos+1]);
+		    *((uint8_t *)args[i].field) |= (uint8_t)atoi(argv[argidx+1]);
 		    break;
 		  case 2:
-		    *((uint16_t *)args[i].field) |= (uint16_t)atoi(argv[pos+1]);
+		    *((uint16_t *)args[i].field) |= (uint16_t)atoi(argv[argidx+1]);
 		    break;
 		  case 4:
-		    *((uint32_t *)args[i].field) |= (uint32_t)atol(argv[pos+1]);
+		    *((uint32_t *)args[i].field) |= (uint32_t)atol(argv[argidx+1]);
 		    break;
 		  case 8:
-		    *((uint64_t *)args[i].field) |= (uint64_t)atol(argv[pos+1]);
+		    *((uint64_t *)args[i].field) |= (uint64_t)atol(argv[argidx+1]);
 		    break;
 		  }
 		  break;
 		case CMDSOCK_FIELD_IPV4:
-		  if (!inet_aton(argv[pos+1], ((struct in_addr *)args[i].field))) {
-		    fprintf(stderr, "Invalid IP Address: %s\n", argv[pos+1]);
+		  if (!inet_aton(argv[argidx+1], ((struct in_addr *)args[i].field))) {
+		    fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx+1]);
 		    return usage(argv[0]);
 		  }
 		  break;
@@ -310,34 +338,22 @@ int main(int argc, char **argv) {
 	    }
 
 	    if (i == count) {
-	      fprintf(stderr, "Unknown argument: %s\n", argv[pos]);
+	      fprintf(stderr, "Unknown argument: %s\n", argv[argidx]);
 	      return usage(argv[0]);
 	    }
 
 	    if (args[i].length) {
-	      argc -= 2;
-	      pos += 2;
+	      c -= 2;
+	      argidx += 2;
 	    } else {
-	      argc --;
-	      pos ++;
+	      c --;
+	      argidx ++;
 	    }
 	  }
 	}
-	break;
-      case CMDSOCK_ENTRY_FOR_IP:
-	{
-	  /* Test for a valid ip argument. */
-  	  if (argc < argidx+1) {
-  	    fprintf(stderr, "%s requires an IP address argument\n", cmd);
-  	    return usage(argv[0]);
-  	  }
-	  
-	  if (!inet_aton(argv[argidx], &request.data.sess.ip)) {
-	    fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx]);
-	    return usage(argv[0]);
-	  }
-	}
-	break;
+	if (request.type != CMDSOCK_LOGOUT || argidx >= argc)
+	  break;
+	/* else, drop through */
       case CMDSOCK_DHCP_DROP:
       case CMDSOCK_DHCP_RELEASE:
       case CMDSOCK_ENTRY_FOR_MAC:
@@ -377,6 +393,21 @@ int main(int argc, char **argv) {
 	  /* do another switch to pick up param configs for authorize */
 	}
 	break;
+      case CMDSOCK_ENTRY_FOR_IP:
+	{
+	  /* Test for a valid ip argument. */
+  	  if (argc < argidx+1) {
+  	    fprintf(stderr, "%s requires an IP address argument\n", cmd);
+  	    return usage(argv[0]);
+  	  }
+	  
+	  if (!inet_aton(argv[argidx], &request.data.sess.ip)) {
+	    fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx]);
+	    return usage(argv[0]);
+	  }
+	}
+	break;
+#ifdef ENABLE_MULTIROUTE
       case CMDSOCK_ROUTE:
       case CMDSOCK_ROUTE_GW:
 	{
@@ -420,6 +451,7 @@ int main(int argc, char **argv) {
 	  /* do another switch to pick up param configs for authorize */
 	}
 	break;
+#endif
       }
       break;
     }
@@ -430,6 +462,35 @@ int main(int argc, char **argv) {
     exit(1);
   }
   
+#ifdef ENABLE_CLUSTER
+  if (peerid > -1) {
+
+    struct sockaddr_in s;
+    int blen = sizeof(struct pkt_chillihdr_t);
+    uint8_t b[blen];
+    int len;
+
+    int fd = socket(AF_INET, SOCK_DGRAM, 0);
+
+    printf("blen %d\n", blen);
+
+    if (fd < 0) {
+      log_err(errno,"socket() failed");
+      exit(1);
+    }
+
+    memset(&s, 0, sizeof(struct sockaddr_in));
+    s.sin_family = AF_INET;
+    s.sin_port = htons(10203);
+    s.sin_addr.s_addr = htonl(INADDR_BROADCAST);
+    
+    len = safe_sendto(fd, b, blen, 0, 
+		      (struct sockaddr *)&s, 
+		      sizeof(struct sockaddr_in));
+
+  } else {
+#endif
+  
 #ifdef HAVE_GLOB
   globbuf.gl_offs = 0;
   glob(cmdsock, GLOB_DOOFFS, NULL, &globbuf);
@@ -444,7 +505,7 @@ int main(int argc, char **argv) {
     if (globbuf.gl_pathc>1) {
       char header[256];
       int headerlen;
-      snprintf(header, sizeof(header), "\nQuerying socket %s\n", cmdsock);
+      safe_snprintf(header, sizeof(header), "\nQuerying socket %s\n", cmdsock);
       headerlen=strlen(header);
       if (write(1, header, strlen(header))!=headerlen) {
         perror("write");
@@ -496,6 +557,10 @@ int main(int argc, char **argv) {
   }
   globfree(&globbuf);
 #endif
+
+#ifdef ENABLE_CLUSTER
+  }
+#endif
   
   return 0;
 }
