@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
  */
 
 #include "chilli.h"
+#include "debug.h"
+#ifdef ENABLE_MODULES
+#include "chilli_module.h"
+#endif
 
 #ifdef USING_MMAP
 #include <sys/mman.h>
@@ -68,7 +72,6 @@ int safe_recvfrom(int sockfd, void *buf, size_t len, int flags,
   return ret;
 }
 
-
 int safe_write(int s, void *b, size_t blen) {
   int ret;
   do {
@@ -241,10 +244,15 @@ int net_open(net_interface *netif) {
   net_close(netif);
   net_gflags(netif);
 
-  if (!(netif->devflags & IFF_UP) || !(netif->devflags & IFF_RUNNING)) {
+  if (
+#ifdef ENABLE_LAYER3
+      !_options.layer3 &&
+#endif
+      ( !(netif->devflags & IFF_UP) || !(netif->devflags & IFF_RUNNING) )) {
     struct in_addr noaddr;
     net_sflags(netif, netif->devflags | IFF_NOARP);
     memset(&noaddr, 0, sizeof(noaddr));
+    log_dbg("removing ip address from %s", netif->devname);
     dev_set_address(netif->devname, &noaddr, NULL, NULL);
   }
 
@@ -262,7 +270,6 @@ static int net_setsockopt(int s, int l, int op, void *v, socklen_t vl) {
   return 0;
 }
 
-
 int net_reopen(net_interface *netif) {
   int previous_fd = netif->fd;
   int option;
@@ -312,36 +319,69 @@ int net_select_init(select_ctx *sctx) {
 }
 
 int net_select_prepare(select_ctx *sctx) {
-#if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
-#else
   int i;
+
+#ifdef ENABLE_MODULES
+  for (i=0; i < MAX_MODULES; i++) {
+    if (!_options.modules[i].name[0]) break;
+    if (_options.modules[i].ctx) {
+      struct chilli_module *m = 
+	(struct chilli_module *)_options.modules[i].ctx;
+      if (m->net_select)
+	m->net_select(sctx); 
+    }
+  }
+#endif
+
+#if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
+  i = 0; /* for compiler */
+#else
 #ifdef USING_POLL
-  for (i=0; i < MAX_SELECT && sctx->desc[i].fd; i++) {
-    sctx->pfds[i].fd = sctx->desc[i].fd;
-    sctx->pfds[i].events = 0;
-    if (sctx->desc[i].evts & SELECT_READ)
-      sctx->pfds[i].events |= POLLIN;
-    if (sctx->desc[i].evts & SELECT_WRITE)
-      sctx->pfds[i].events |= POLLOUT;
+  for (i=0; i < MAX_SELECT; i++) {
+    if (sctx->desc[i].fd) {
+      sctx->pfds[i].fd = sctx->desc[i].fd;
+      sctx->pfds[i].events = 0;
+      if (sctx->desc[i].evts & SELECT_READ)
+	sctx->pfds[i].events |= POLLIN;
+      if (sctx->desc[i].evts & SELECT_WRITE)
+	sctx->pfds[i].events |= POLLOUT;
+    }
   }
 #else
   fd_zero(&sctx->rfds);
   fd_zero(&sctx->wfds);
   fd_zero(&sctx->efds);
-  for (i=0; i < MAX_SELECT && sctx->desc[i].fd; i++) {
-    if (sctx->desc[i].evts & SELECT_READ)
-      fd_set(sctx->desc[i].fd, &sctx->rfds);
-    if (sctx->desc[i].evts & SELECT_WRITE)
-      fd_set(sctx->desc[i].fd, &sctx->wfds);
+  for (i=0; i < MAX_SELECT; i++) {
+    if (!sctx->desc[i].fd && sctx->desc[i].evts & SELECT_RESET) {
+      sctx->desc[i].cb(&sctx->desc[i], -1);
+    }
+    if (sctx->desc[i].fd) {
+      if (sctx->desc[i].evts & SELECT_READ)
+	fd_set(sctx->desc[i].fd, &sctx->rfds);
+      if (sctx->desc[i].evts & SELECT_WRITE)
+	fd_set(sctx->desc[i].fd, &sctx->wfds);
+    }
   }
 #endif
 #endif
   return 0;
 }
 
+int net_select_dereg(select_ctx *sctx, int oldfd) {
+  int i;
+  for (i=0; i < sctx->count; i++) {
+    if (sctx->desc[i].fd == oldfd) {
+      memset(&sctx->desc[i], 0, sizeof(select_fd));
+      sctx->count--;
+      return 0;
+    }
+  }
+  return -1;
+}
+
 int net_select_rereg(select_ctx *sctx, int oldfd, int newfd) {
   int i;
-  for (i=0; i<sctx->count; i++) {
+  for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd == oldfd) {
       sctx->desc[i].fd = newfd;
 #if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
@@ -396,6 +436,7 @@ int net_select_reg(select_ctx *sctx, int fd, char evts,
   if (fd > sctx->maxfd) sctx->maxfd = fd;
 #endif
   sctx->count++;
+  log_dbg("net select count: %d", sctx->count);
   return 0;
 }
 
@@ -405,13 +446,15 @@ int net_select_zero(select_ctx *sctx) {
 #else
 #ifdef USING_POLL
   memset(&sctx->pfds, 0, sizeof(sctx->pfds));
-  for (i=0; i < MAX_SELECT && sctx->desc[i].fd; i++) {
-    sctx->pfds[i].fd = sctx->desc[i].fd;
-    sctx->pfds[i].events = 0;
-    if (sctx->desc[i].evts & SELECT_READ)
-      sctx->pfds[i].events |= POLLIN;
-    if (sctx->desc[i].evts & SELECT_WRITE)
-      sctx->pfds[i].events |= POLLOUT;
+  for (i=0; i < MAX_SELECT; i++) {
+    if (sctx->desc[i].fd) {
+      sctx->pfds[i].fd = sctx->desc[i].fd;
+      sctx->pfds[i].events = 0;
+      if (sctx->desc[i].evts & SELECT_READ)
+	sctx->pfds[i].events |= POLLIN;
+      if (sctx->desc[i].evts & SELECT_WRITE)
+	sctx->pfds[i].events |= POLLOUT;
+    }
   }
 #else
   fd_zero(&sctx->rfds);
@@ -571,14 +614,16 @@ int net_run_selected(select_ctx *sctx, int status) {
     sfd->cb(sfd->ctx, sfd->idx);
   }
 #else
-  for (i=0; i < MAX_SELECT && sctx->desc[i].fd; i++) {
+  for (i=0; i < MAX_SELECT; i++) {
+    if (sctx->desc[i].fd) {
 #ifdef USING_POLL
-    char has_read = !!(sctx->pfds[i].revents & POLLIN);
+      char has_read = !!(sctx->pfds[i].revents & POLLIN);
 #else
-    char has_read = fd_isset(sctx->desc[i].fd, &sctx->rfds);
+      char has_read = fd_isset(sctx->desc[i].fd, &sctx->rfds);
 #endif
-    if (has_read) {
-      sctx->desc[i].cb(sctx->desc[i].ctx, sctx->desc[i].idx);
+      if (has_read) {
+	sctx->desc[i].cb(sctx->desc[i].ctx, sctx->desc[i].idx);
+      }
     }
   }
 #endif
@@ -691,10 +736,15 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 #endif
 
   if (netif->fd) {
+    int addr_len;
+    struct sockaddr_ll s_addr;
+    memset (&s_addr, 0, sizeof (struct sockaddr_ll));
+    addr_len = sizeof (s_addr);
 
     len = safe_recvfrom(netif->fd, d, dlen,
 			MSG_DONTWAIT | MSG_TRUNC, 
-			NULL, NULL);
+			(struct sockaddr *) &s_addr, 
+			(socklen_t *) &addr_len);
 
     /*len = safe_read(netif->fd, d, dlen);*/
 
@@ -986,7 +1036,9 @@ int net_open_eth(net_interface *netif) {
   memset(&ifr, 0, sizeof(ifr));
 
   /* Create socket */
-  if ((netif->fd = socket(PF_PACKET, SOCK_RAW, htons(netif->protocol))) < 0) {
+  if ((netif->fd = socket(PF_PACKET, 
+			  netif->idx ? SOCK_DGRAM : SOCK_RAW, 
+			  htons(netif->protocol))) < 0) {
     if (errno == EPERM) {
       log_err(errno, "Cannot create raw socket. Must be root.");
     }
@@ -1003,12 +1055,14 @@ int net_open_eth(net_interface *netif) {
   coe(netif->fd);
 
   option = 1;
-  if (net_setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, &option, sizeof(option)) < 0)
+  if (net_setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, 
+		     &option, sizeof(option)) < 0)
     return -1;
 
   /* Enable reception and transmission of broadcast frames */
   option = 1;
-  if (net_setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, &option, sizeof(option)) < 0)
+  if (net_setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, 
+		     &option, sizeof(option)) < 0)
     return -1;
 
   if (_options.sndbuf > 0) {
@@ -1103,7 +1157,8 @@ int net_open_eth(net_interface *netif) {
     mr.mr_ifindex = netif->ifindex;
     mr.mr_type = PACKET_MR_PROMISC;
 
-    if (net_setsockopt(netif->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (char *)&mr, sizeof(mr)) < 0)
+    if (net_setsockopt(netif->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, 
+		       (char *)&mr, sizeof(mr)) < 0)
       return -1;
   }
 #endif
