@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -153,6 +153,7 @@ struct queue_item
 
 #define SELECT_READ 1
 #define SELECT_WRITE 2
+#define SELECT_RESET 4
 
 typedef void (*select_callback)(void *data, int idx);
 
@@ -195,9 +196,16 @@ typedef struct _net_interface {
 
   /* network/address */
   struct in_addr address;
-  struct in_addr gateway;
+  struct in_addr network;
   struct in_addr netmask;
   struct in_addr broadcast;
+  struct in_addr gateway;
+
+#ifdef ENABLE_MULTIROUTE
+#if(1) /* XXX  ONE2ONE */
+  struct in_addr nataddress;
+#endif
+#endif
 
   /* socket/descriptor */
   int fd;
@@ -244,6 +252,8 @@ typedef struct _net_interface {
 #define NET_PROMISC (1<<0)
 #define NET_USEMAC  (1<<1)
 #define NET_ETHHDR  (1<<2)
+#define NET_PPPHDR  (1<<3)
+
 } net_interface;
 
 typedef int (*net_handler)(void *ctx, void *data, size_t len);
@@ -264,6 +274,9 @@ int net_init(net_interface *netif, char *ifname, uint16_t protocol, int promisc,
 int net_route(struct in_addr *dst, struct in_addr *gateway, struct in_addr *mask, int delete);
 int net_set_mtu(net_interface *netif, size_t mtu);
 
+int safe_sendto(int s, const void *b, size_t blen, int flags,
+		const struct sockaddr *dest_addr, socklen_t addrlen);
+
 int net_select_init(select_ctx *sctx);
 int net_select_prepare(select_ctx *sctx);
 int net_select(select_ctx *sctx);
@@ -288,6 +301,7 @@ int net_open_nfqueue(net_interface *netif, uint16_t q, int (*cb)());
 
 int net_select_reg(select_ctx *sctx, int fd, char evts, select_callback cb, void *ctx, int idx);
 int net_select_rereg(select_ctx *sctx, int oldfd, int newfd);
+int net_select_dereg(select_ctx *sctx, int oldfd);
 
 int net_close(net_interface *netif);
 
