@@ -1,7 +1,7 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
  * Copyright (C) 2006 PicoPoint B.V.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
  */
 
 #include "chilli.h"
+#ifdef ENABLE_MODULES
+#include "chilli_module.h"
+#endif
 
 void options_init() {
   memset(&_options, 0, sizeof(_options));
@@ -50,7 +53,7 @@ int option_aton(struct in_addr *addr, struct in_addr *mask,
 
   switch (c) {
   case 4:
-    mask->s_addr = 0xffffffff;
+    mask->s_addr = htonl(0xffffff00);
     break;
   case 5:
     if (m1 > 32) {
@@ -144,7 +147,6 @@ static int opt_run(int argc, char **argv, int reload) {
   exit(0);
 }
 
-
 int options_load(int argc, char **argv, bstring bt) {
   static char done_before = 0;
   char file[128];
@@ -161,16 +163,33 @@ int options_load(int argc, char **argv, bstring bt) {
     if (WIFEXITED(status) && WEXITSTATUS(status) == 2) exit(0);
     fd = open(file, O_RDONLY);
     if (fd <= 0) {
-      log_warn(0, "could not generate configuration (%s), sleeping one second", file);
       if (done_before) break;
-      sleep(1);
+      else {
+	char *offline = getenv("CHILLI_OFFLINE");
+
+	if (offline) {
+
+	  execl(
+#ifdef ENABLE_CHILLISCRIPT
+		SBINDIR "/chilli_script", SBINDIR "/chilli_script", _options.binconfig, 
+#else
+		offline,
+#endif
+		offline, (char *) 0);
+
+	  break;
+	} 
+
+	log_warn(0, "could not generate configuration (%s), sleeping one second", file);
+	sleep(1);
+      }
     }
   }
 
   if (fd <= 0) return 0;
   done_before = 1;
 
-  log_dbg("rereading binary file %s", file);
+  log_dbg("PID %d rereading binary file %s", getpid(), file);
   return options_fromfd(fd, bt);
 }
 
@@ -250,6 +269,8 @@ int options_fromfd(int fd, bstring bt) {
   if (!option_s_l(bt, &o.ipdown)) return 0;
   if (!option_s_l(bt, &o.conup)) return 0;
   if (!option_s_l(bt, &o.condown)) return 0;
+  if (!option_s_l(bt, &o.macup)) return 0;
+  if (!option_s_l(bt, &o.macdown)) return 0;
 
   if (!option_s_l(bt, &o.radiussecret)) return 0;
   if (!option_s_l(bt, &o.radiusnasid)) return 0;
@@ -260,6 +281,7 @@ int options_fromfd(int fd, bstring bt) {
   
   if (!option_s_l(bt, &o.dhcpif)) return 0;
   if (!option_s_l(bt, &o.routeif)) return 0;
+  if (!option_s_l(bt, &o.peerkey)) return 0;
 
   if (!option_s_l(bt, &o.macsuffix)) return 0;
   if (!option_s_l(bt, &o.macpasswd)) return 0;
@@ -286,6 +308,18 @@ int options_fromfd(int fd, bstring bt) {
 #ifdef HAVE_NETFILTER_COOVA
   if (!option_s_l(bt, &o.kname)) return 0;
 #endif
+#ifdef ENABLE_DNSLOG
+  if (!option_s_l(bt, &o.dnslog)) return 0;
+#endif
+#ifdef ENABLE_IPWHITELIST
+  if (!option_s_l(bt, &o.ipwhitelist)) return 0;
+#endif
+#ifdef ENABLE_UAMDOMAINFILE
+  if (!option_s_l(bt, &o.uamdomainfile)) return 0;
+#endif
+#ifdef ENABLE_MODULES
+  if (!option_s_l(bt, &o.moddir)) return 0;
+#endif
 
   if (!option_s_l(bt, &o.adminuser)) return 0;
   if (!option_s_l(bt, &o.adminpasswd)) return 0;
@@ -317,10 +351,41 @@ int options_fromfd(int fd, bstring bt) {
   }
 #endif
 
+#ifdef ENABLE_MODULES
+  for (i=0; i < MAX_MODULES; i++) {
+    if (!_options.modules[i].name[0]) break;
+    if (!_options.modules[i].ctx) continue;
+    else {
+      struct chilli_module *m = 
+	(struct chilli_module *)_options.modules[i].ctx;
+      if (m->destroy)
+	m->destroy();
+    }
+    log_dbg("Unloading module %s",_options.modules[i].name);
+    chilli_module_unload(_options.modules[i].ctx);
+  }
+#endif
+
   if (_options._data) free(_options._data);
   memcpy(&_options, &o, sizeof(o));
   _options._data = (char *)bt->data;
 
+#ifdef ENABLE_MODULES
+  log_dbg("Loading modules");
+  for (i=0; i < MAX_MODULES; i++) {
+    if (!_options.modules[i].name[0]) break;
+    log_dbg("Loading module %s",_options.modules[i].name);
+    chilli_module_load(&_options.modules[i].ctx, 
+		       _options.modules[i].name);
+    if (_options.modules[i].ctx) {
+      struct chilli_module *m = 
+	(struct chilli_module *)_options.modules[i].ctx;
+      if (m->initialize)
+	m->initialize(_options.modules[i].conf); 
+    }
+  }
+#endif
+
   /* 
    *  We took the buffer and this bt will be destroyed.
    *  Give the bstring a bogus buffer so that bdestroy() works.
@@ -338,6 +403,8 @@ int options_save(char *file, bstring bt) {
   mode_t oldmask;
   int fd, i;
 
+  log_dbg("PID %d saving options to %s", getpid(), file);
+
   memcpy(&o, &_options, sizeof(o));
 
 #ifdef ENABLE_CHILLIREDIR
@@ -362,6 +429,8 @@ int options_save(char *file, bstring bt) {
   if (!option_s_s(bt, &o.ipdown)) return 0;
   if (!option_s_s(bt, &o.conup)) return 0;
   if (!option_s_s(bt, &o.condown)) return 0;
+  if (!option_s_s(bt, &o.macup)) return 0;
+  if (!option_s_s(bt, &o.macdown)) return 0;
 
   if (!option_s_s(bt, &o.radiussecret)) return 0;
   if (!option_s_s(bt, &o.radiusnasid)) return 0;
@@ -372,6 +441,7 @@ int options_save(char *file, bstring bt) {
 
   if (!option_s_s(bt, &o.dhcpif)) return 0;
   if (!option_s_s(bt, &o.routeif)) return 0;
+  if (!option_s_s(bt, &o.peerkey)) return 0;
 
   if (!option_s_s(bt, &o.macsuffix)) return 0;
   if (!option_s_s(bt, &o.macpasswd)) return 0;
@@ -398,6 +468,18 @@ int options_save(char *file, bstring bt) {
 #ifdef HAVE_NETFILTER_COOVA
   if (!option_s_s(bt, &o.kname)) return 0;
 #endif
+#ifdef ENABLE_DNSLOG
+  if (!option_s_s(bt, &o.dnslog)) return 0;
+#endif
+#ifdef ENABLE_IPWHITELIST
+  if (!option_s_s(bt, &o.ipwhitelist)) return 0;
+#endif
+#ifdef ENABLE_UAMDOMAINFILE
+  if (!option_s_s(bt, &o.uamdomainfile)) return 0;
+#endif
+#ifdef ENABLE_MODULES
+  if (!option_s_s(bt, &o.moddir)) return 0;
+#endif
 
   if (!option_s_s(bt, &o.adminuser)) return 0;
   if (!option_s_s(bt, &o.adminpasswd)) return 0;
@@ -465,6 +547,7 @@ int options_binload(char *file) {
   int ok = 0;
   if (fd > 0) {
     bstring bt = bfromcstr("");
+    log_dbg("PID %d loading binary options file %s", getpid(), file);
     ok = options_fromfd(fd, bt);
     bdestroy(bt);
     return ok;
@@ -493,7 +576,6 @@ int process_options(int argc, char **argv, int minimal) {
   }
   
   umask(process_mask);
-  
   return !reload_options(argc, argv);
 }
 
@@ -504,7 +586,7 @@ void reprocess_options(int argc, char **argv) {
 int reload_options(int argc, char **argv) {
   bstring bt = bfromcstr("");
   int ok = options_load(argc, argv, bt);
-  log_dbg("reloaded binary options file");
+  log_dbg("PID %d reloaded binary options file", getpid());
   bdestroy(bt);
   return ok;
 }
@@ -513,3 +595,12 @@ void options_destroy() {
   if (_options._data) 
     free(_options._data);
 }
+
+void options_cleanup() {
+  char file[128];
+  chilli_binconfig(file, sizeof(file), getpid());
+  log_dbg("Removing %s", file);
+  if (remove(file)) log_dbg("remove(%s) failed", file);
+  options_destroy();
+}
+
