@@ -1,7 +1,7 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
  * Copyright (C) 2006 PicoPoint B.V.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +57,8 @@ struct options_t {
   char * ipdown;                 /* Script to run after link-down */
   char * conup;                  /* Script to run after session/connection-up */
   char * condown;                /* Script to run after session/connection-down */
+  char * macup;
+  char * macdown;
   int txqlen;
 
   int ringsize;
@@ -98,9 +100,13 @@ struct options_t {
   uint8_t dhcpmac[PKT_ETH_ALEN]; /* Interface MAC address */
   struct in_addr dhcplisten;     /* IP address to listen to */
   int lease;                     /* DHCP lease time */
+  int leaseplus;                 /* DHCP lease grace period */
   int dhcpstart;
   int dhcpend;
 
+  uint8_t peerid;
+  char *peerkey;
+
   /* XXX */
   uint8_t nexthop[PKT_ETH_ALEN];
 #ifdef ENABLE_NETNAT
@@ -138,6 +144,7 @@ struct options_t {
   char *uamhostname;             /* Simple hostname (no dots) DNS name for uamlisten */
 
   /* booleans */
+  uint8_t layer3;                   /* Layer3 only support */
   uint8_t allowdyn:1;               /* Allow dynamic address allocation */
   uint8_t allowstat:1;              /* Allow static address allocation */
   uint8_t dhcpusemac:1;             /* Use given MAC or interface default */
@@ -169,13 +176,18 @@ struct options_t {
   uint8_t dhcp_broadcast:1;         /* always broadcast DHCP (when not relaying) */
   uint8_t seskeepalive:1;           /* Keep sessions alive during shutdown */
   uint8_t strictmacauth:1;          /* Be strict about DHCP macauth (don't reply DHCP until we get RADIUS) */
+  uint8_t strictdhcp:1;             /* Be strict about DHCP allocating from dyn-pool only */
   uint8_t dhcpmacset:1;             /* Set the dhcpif interface with the dhcpmac */
   uint8_t uamallowpost:1;           /* Set to true if the UAMPORT is allowed to access a POST */
+  uint8_t redir:1;                  /* Launch redir sub-process instead of forking */
   uint8_t redirssl:1;               /* Enable redirection of SSL/HTTPS port (requires SSL support) */
   uint8_t uamuissl:1;               /* Enable SSL/HTTPS on uamuiport (requires SSL support) */
   uint8_t domaindnslocal:1;         /* Wildcard option to consider all hostnames *.domain local */
   uint8_t radsec:1;                 /* Use RadSec tunneling */
   uint8_t proxymacaccept:1;         /* Auto-accept non-EAP requests on proxy port */
+  uint8_t noradallow:1;             /* Authorize all sessions when RADIUS is not available */
+  uint8_t redirdnsreq:1;
+  uint8_t routeonetone:1;
   /* */
 
   pass_through pass_throughs[MAX_PASS_THROUGHS];
@@ -236,6 +248,23 @@ struct options_t {
   char *nasmac;
   char *nasip;
 
+#ifdef ENABLE_DHCPOPT
+  uint8_t dhcp_options[512];
+  int dhcp_options_len;
+#endif
+
+#ifdef ENABLE_DNSLOG
+  char *dnslog;
+#endif
+
+#ifdef ENABLE_IPWHITELIST
+  char *ipwhitelist;
+#endif
+
+#ifdef ENABLE_UAMDOMAINFILE
+  char *uamdomainfile;
+#endif
+
   /* Command-Socket */
   char *cmdsocket;
 
@@ -253,6 +282,16 @@ struct options_t {
   char *kname;
 #endif
 
+#ifdef ENABLE_MODULES
+#define MAX_MODULES 4
+  struct {
+    char name[32];
+    char conf[128];
+    void *ctx;
+  } modules[MAX_MODULES];
+  char *moddir;
+#endif
+
   char * _data; /* actual data buffer for loaded options */
 };
 
@@ -266,6 +305,7 @@ int reload_options(int argc, char **argv);
 int options_save(char *file, bstring bt);
 void options_init();
 void options_destroy();
+void options_cleanup();
 
 #ifndef MAIN_FILE /* all main() files must implement _options */
 extern struct options_t _options;
