@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,8 @@ struct session_params {
 #define REQUIRE_REDIRECT   (1<<2)
 #define IS_UAM_REAUTH      (1<<3)
 #define NO_ACCOUNTING      (1<<4)
+#define NO_SCRIPT          (1<<5)
+#define ADMIN_LOGIN        (1<<6)
   uint8_t flags;
 
 #ifdef ENABLE_SESSGARDEN
@@ -59,11 +61,23 @@ struct redir_state {
 
   uint8_t uamchal[REDIR_MD5LEN];
 
+  /* To store the RADIUS CLASS attribute received in the Access Accept */
   uint8_t classbuf[RADIUS_ATTR_VLEN];
   size_t classlen;
 
+  /* To store the RADIUS CUI attribute received in the Access Accept */
+  uint8_t cuibuf[RADIUS_ATTR_VLEN];
+  size_t cuilen;
+  
+  /* To store the RADIUS STATE attribute between Radius requests */
   uint8_t statebuf[RADIUS_ATTR_VLEN];
-  unsigned char statelen;
+  uint8_t statelen;
+
+  /*  EAP identity of the last request sent */
+  uint8_t eap_identity;
+
+  /* UAM protocol used */
+  uint8_t uamprotocol;
 
 #ifdef ENABLE_PROXYVSA
 #define RADIUS_PROXYVSA 128
@@ -108,12 +122,4 @@ struct session_state {
 
 } __attribute__((packed));
 
-
-int session_json_fmt(struct session_state *state, 
-		     struct session_params *params,
-		     bstring json, int init);
-
-int session_redir_json_fmt(bstring json, char *userurl, char *redirurl, 
-			   bstring logouturl, uint8_t *hismac);
-
 #endif
