@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -210,6 +210,13 @@
 #include <grp.h>
 #endif
 
+#include <sys/types.h>
+#include <dirent.h>
+
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif
+
 #undef LITTLE_ENDIAN
 #undef BIG_ENDIAN
 
@@ -234,6 +241,12 @@
 #include <unistd.h>
 #include <errno.h>
 
+#define safe_snprintf portable_snprintf
+char *safe_strncpy(char *dst, const char *src, size_t size);
+
+/* XXX */
+#define ENABLE_IPV6 1
+
 #ifndef TEMP_FAILURE_RETRY
 #define TEMP_FAILURE_RETRY(expression) \
     ({ \
@@ -248,10 +261,6 @@
     memset ((char *) &(addr), '\0', sizeof(addr));	\
     addr.sa_family = (family);
 
-#include "bstrlib.h"
-
-int bstring_fromfd(bstring s, int fd);
-
 void copy_mac6(uint8_t *, uint8_t *);
 
 #ifndef USING_IPC_MSG
