@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,11 +28,114 @@
  */
 
 #include "chilli.h"
+#include "debug.h"
+#ifdef ENABLE_MULTIROUTE
+#include "rtmon.h"
+extern struct rtmon_t _rtmon;
+#endif
 
 #define inaddr(x)    (((struct sockaddr_in *)&ifr->x)->sin_addr)
 #define inaddr2(p,x) (((struct sockaddr_in *)&(p)->x)->sin_addr)
 
-#define _debug_ 1
+#ifdef ENABLE_MULTIROUTE
+net_interface * tun_nextif(struct tun_t *tun) {
+  net_interface *netif;
+  int i;
+
+  if (tun->_interface_count == TUN_MAX_INTERFACES)
+    return 0;
+
+  for (i=0; i<TUN_MAX_INTERFACES; i++) {
+    netif = &tun->_interfaces[i];
+    if (!netif->ifindex && !netif->fd) {
+      if (!netif->idx)
+	netif->idx = tun->_interface_count;
+      tun->_interface_count++;
+      return netif;
+    }
+  }
+
+  return 0;
+}
+
+net_interface * tun_newif(struct tun_t *tun, net_interface *netif) {
+  net_interface *newif = tun_nextif(tun);
+      
+  if (newif) {
+    int idx = newif->idx;
+    memcpy(newif, netif, sizeof(net_interface));
+    newif->idx = idx;
+
+    if (newif->devflags & IFF_POINTOPOINT)
+      newif->flags |= NET_PPPHDR | NET_ETHHDR;
+  }
+
+  return newif;
+}
+
+void tun_delif(struct tun_t *tun, int ifindex) {
+  net_interface *netif;
+  int i;
+
+  for (i=0; i<TUN_MAX_INTERFACES; i++) {
+    netif = &tun->_interfaces[i];
+    if (netif->ifindex == ifindex) {
+      int idx = netif->idx;
+      net_select_dereg(tun->sctx, netif->fd);
+      net_close(netif);
+      memset(netif, 0, sizeof(net_interface));
+      netif->idx = idx;
+      tun->_interface_count--;
+      return;
+    }
+  }
+}
+
+int tun_name2idx(struct tun_t *tun, char *name) {
+  int i;
+
+  for (i=0; i<TUN_MAX_INTERFACES; i++) 
+    if (!strcmp(name, tun->_interfaces[i].devname))
+      return tun->_interfaces[i].idx;
+
+  /* Not found? Check for discovery */
+  {
+    struct rtmon_iface *rti = rtmon_find(&_rtmon, name);
+    if (rti) {
+      net_interface *newif = 0;
+      net_interface netif;
+      log_dbg("Discoving TUN %s", name);
+      memset(&netif, 0, sizeof(netif));
+      safe_strncpy(netif.devname, rti->devname, sizeof(netif.devname));
+      memcpy(netif.hwaddr, rti->hwaddr, sizeof(netif.hwaddr));
+      netif.address = rti->address;
+      netif.netmask = rti->netmask;
+      netif.gateway = rti->gateway;
+      netif.broadcast = rti->broadcast;
+      netif.devflags = rti->devflags;
+      netif.mtu = rti->mtu;
+      netif.ifindex = rti->index;
+      
+      newif = tun_newif(tun, &netif);
+      if (newif) {
+
+	if (net_init(newif, 0, ETH_P_ALL, 1, 0) < 0) {
+	  log_err(errno, "net_init");
+	}
+	else {
+	  net_select_reg(tun->sctx, 
+			 newif->fd,
+			 SELECT_READ, (select_callback) tun_decaps, 
+			 tun, newif->idx);
+	}
+	
+	return newif->idx;
+      }
+    }
+  }
+
+  return 0; /* tun/tap index */
+}
 
 int tun_discover(struct tun_t *this) {
   net_interface netif;
@@ -100,14 +203,15 @@ int tun_discover(struct tun_t *this) {
 
     } else log_err(errno, "ioctl(SIOCGIFNETMASK)");
 
-
     /* hardware address */
 #ifdef SIOCGIFHWADDR
     if (-1 < ioctl(fd, SIOCGIFHWADDR, (caddr_t)ifr)) {
       switch (ifr->ifr_hwaddr.sa_family) {
+      case  ARPHRD_PPP:
+	netif.flags |= NET_PPPHDR | NET_ETHHDR;
+	break;
       case  ARPHRD_NETROM:  
       case  ARPHRD_ETHER:  
-      case  ARPHRD_PPP:
       case  ARPHRD_EETHER:  
       case  ARPHRD_IEEE802: 
 	{
@@ -147,6 +251,7 @@ int tun_discover(struct tun_t *this) {
     if (netif.devflags & IFF_POINTOPOINT) {
       if (-1 < ioctl(fd, SIOCGIFDSTADDR, (caddr_t)ifr)) {
 
+	netif.flags |= NET_PPPHDR;
 	netif.gateway = inaddr(ifr_addr);
 	log_dbg("\tPoint-to-Point:\t%s", inet_ntoa(inaddr(ifr_dstaddr)));
 
@@ -179,12 +284,9 @@ int tun_discover(struct tun_t *this) {
       continue;
 
     {
-      net_interface *newif = tun_nextif(tun);
+      net_interface *newif = tun_newif(tun, &netif);
       
       if (newif) {
-	int idx = newif->idx;
-	memcpy(newif, &netif, sizeof(netif));
-	newif->idx = idx;
 
 	if (net_init(newif, 0, ETH_P_ALL, 1, 0) < 0) {
 	  log_err(errno, "net_init");
@@ -203,30 +305,8 @@ int tun_discover(struct tun_t *this) {
   close(fd);
   return 0;
 }
+#endif
 
-net_interface * tun_nextif(struct tun_t *tun) {
-  net_interface *netif;
-
-  if (tun->_interface_count == TUN_MAX_INTERFACES)
-    return 0;
-
-  netif = &tun->_interfaces[tun->_interface_count];
-  netif->idx = tun->_interface_count;
-
-  tun->_interface_count++;
-
-  return netif;
-}
-
-int tun_name2idx(struct tun_t *tun, char *name) {
-  int i;
-
-  for (i=0; i<tun->_interface_count; i++) 
-    if (!strcmp(name, tun->_interfaces[i].devname))
-      return i;
-
-  return 0; /* tun/tap index */
-}
 
 #if defined(__linux__)
 
@@ -480,20 +560,24 @@ int tuntap_interface(struct _net_interface *netif) {
   memset(&ifr, 0, sizeof(ifr));
 
   /* Tun device, no packet info */
-  ifr.ifr_flags = (_options.usetap ? IFF_TAP : IFF_TUN) | IFF_NO_PI; 
+  ifr.ifr_flags = (
+#ifdef ENABLE_TAP
+		   _options.usetap ? IFF_TAP :
+#endif
+		   IFF_TUN) | IFF_NO_PI; 
   
   ifr.ifr_flags = ifr.ifr_flags
 #ifdef IFF_MULTICAST
-    | IFF_MULTICAST
+     | IFF_MULTICAST
 #endif
 #ifdef IFF_BROADCAST
-    | IFF_BROADCAST
+     | IFF_BROADCAST
 #endif
 #ifdef IFF_PROMISC
-    | IFF_PROMISC
+     | IFF_PROMISC
 #endif
-#if defined(IFF_ONE_QUEUE) && defined(SIOCSIFTXQLEN)
-    | IFF_ONE_QUEUE
+#ifdef IFF_ONE_QUEUE
+     | IFF_ONE_QUEUE
 #endif
     ;
 
@@ -533,6 +617,7 @@ int tuntap_interface(struct _net_interface *netif) {
   ioctl(netif->fd, TUNSETNOCSUM, 1); /* Disable checksums */
 
   /* Get the MAC address of our tap interface */
+#ifdef ENABLE_TAP
   if (_options.usetap) {
     int fd;
     netif->flags |= NET_ETHHDR;
@@ -549,6 +634,7 @@ int tuntap_interface(struct _net_interface *netif) {
       close(fd);
     }
   }
+#endif
   
   return 0;
   
@@ -649,7 +735,6 @@ int tuntap_interface(struct _net_interface *netif) {
 #else
 #error  "Unknown platform!"
 #endif
-
 }
 
 int tun_new(struct tun_t **ptun) {
@@ -660,11 +745,15 @@ int tun_new(struct tun_t **ptun) {
     return EOF;
   }
 
+#ifdef ENABLE_MULTIROUTE
   tuntap_interface(tun_nextif(tun));
 
   if (_options.routeif) {
     tun_discover(tun);
   }
+#else
+  tuntap_interface(&tun->_tuntap);
+#endif
 
   return 0;
 }
@@ -701,6 +790,8 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
 
   char ethhdr = (tun(c->this, c->idx).flags & NET_ETHHDR) != 0;
 
+  if (c->idx) ethhdr = 0;
+
 #if(_debug_)
   log_dbg("tun_decaps(idx=%d, len=%d)", tun(c->this, c->idx).ifindex, length);
 #endif
@@ -739,23 +830,38 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
 	return -1;
     }
 #endif
+
+#ifdef ENABLE_MULTIROUTE
+    if (_options.routeonetone) {
+      iph->daddr = tun(c->this, c->idx).nataddress.s_addr;
+      chksum(iph);
+    }
+#endif
+
     if ((iph->daddr & _options.mask.s_addr) != _options.net.s_addr) {
+#if(_debug_)
+      struct in_addr addr;
+      addr.s_addr = iph->daddr;
+      log_dbg("pkt not for our network %s",inet_ntoa(addr));
+#endif
       return -1;
     }
   } 
   
-  if (iph->version_ihl != PKT_IP_VER_HLEN) {
+  if (!_options.usetap) {
+    if (iph->version_ihl != PKT_IP_VER_HLEN) {
 #if(_debug_)
-    log_dbg("dropping non-IPv4");
+      log_dbg("dropping non-IPv4");
 #endif
-    return -1;
-  }
-
-  if ((int)ntohs(iph->tot_len) + ethsize > length) {
-    log_dbg("dropping ip packet; ip-len=%d + eth-hdr=%d > read-len=%d",
-	    (int)ntohs(iph->tot_len),
-	    ethsize, (int)length);
-    return -1;
+      return -1;
+    }
+    
+    if ((int)ntohs(iph->tot_len) + ethsize > length) {
+      log_dbg("dropping ip packet; ip-len=%d + eth-hdr=%d > read-len=%d",
+	      (int)ntohs(iph->tot_len),
+	      ethsize, (int)length);
+      return -1;
+    }
   }
 
   return c->this->cb_ind(c->this, packet, length, c->idx);
@@ -769,8 +875,13 @@ int tun_decaps(struct tun_t *this, int idx) {
   
   c.this = this;
   c.idx = idx;
-  
-  if ((length = net_read_dispatch(&tun(this, idx), tun_decaps_cb, &c)) < 0) 
+
+  if (idx > 0) 
+    length = net_read_dispatch_eth(&tun(this, idx), tun_decaps_cb, &c);
+  else
+    length = net_read_dispatch(&tun(this, idx), tun_decaps_cb, &c);
+
+  if (length < 0) 
     return -1;
   
   return length;
@@ -835,8 +946,19 @@ int tun_write(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 
 #elif defined(__linux__) || defined (__FreeBSD__) || defined (__APPLE__) || defined (__NetBSD__)
 
-  /*if (idx > 0) {
-    }*/
+#ifdef ENABLE_MULTIROUTE
+  if (idx > 0 && tun(tun, idx).flags & NET_PPPHDR) {
+    struct sockaddr_ll addr;
+    size_t ethlen = sizeofeth(pack);
+    memset(&addr,0,sizeof(addr));
+    addr.sll_family = AF_PACKET;
+    addr.sll_protocol = ethhdr(pack)->prot;
+    addr.sll_ifindex = tun(tun, idx).ifindex;
+    pack += ethlen;
+    len  -= ethlen;
+    return net_write_eth(&tun(tun, idx), pack, len, &addr);
+  }
+#endif
 
   return safe_write(tun(tun, idx).fd, pack, len);
 
@@ -859,6 +981,7 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   if (_options.tcpmss)
     pkt_shape_tcpmss(pack, &len);
 
+#ifdef ENABLE_MULTIROUTE
   if (idx > 0) {
     struct pkt_iphdr_t *iph = iphdr(pack);
     if ((iph->daddr & _options.mask.s_addr) == _options.net.s_addr ||
@@ -868,6 +991,15 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
     }
   }
 
+  if (_options.routeonetone && idx > 0) {
+    struct pkt_iphdr_t *iph = iphdr(pack);
+    if (!tun(tun, idx).nataddress.s_addr)
+      tun(tun, idx).nataddress.s_addr = iph->saddr;
+    iph->saddr = tun(tun, idx).address.s_addr;
+    chksum(iph);
+  }
+#endif
+
 #ifdef ENABLE_NETNAT
   if (idx > 0) {
     if (nat_do(tun, idx, pack, len)) {
@@ -896,7 +1028,6 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
      * TODO: When using ieee8021q, the vlan tag has to be stripped
      * off for the non-vlan WAN.
      */
-
     if (gwaddr[0] == 0 && gwaddr[1] == 0 && gwaddr[2] == 0 && 
 	gwaddr[3] == 0 && gwaddr[4] == 0 && gwaddr[5] == 0) {
       /*  
@@ -914,12 +1045,14 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 
     copy_mac6(ethh->dst, gwaddr);
 
-    /*
-    if (_options.debug)
-      log_dbg("writing to tap src=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x dst=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x len=%d",
-	      ethh->src[0],ethh->src[1],ethh->src[2],ethh->src[3],ethh->src[4],ethh->src[5],
-	      ethh->dst[0],ethh->dst[1],ethh->dst[2],ethh->dst[3],ethh->dst[4],ethh->dst[5], len);
-    */
+#if(_debug_)
+    log_dbg("writing to tap src=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x "
+	    "dst=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x len=%d",
+	    ethh->src[0],ethh->src[1],ethh->src[2],
+	    ethh->src[3],ethh->src[4],ethh->src[5],
+	    ethh->dst[0],ethh->dst[1],ethh->dst[2],
+	    ethh->dst[3],ethh->dst[4],ethh->dst[5], len);
+#endif
     
   } else {
     size_t ethlen = sizeofeth(pack);
@@ -927,7 +1060,9 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
     len  -= ethlen;
   }
 
-  /*log_dbg("tun_encaps(%s) len=%d", tun(tun,idx).devname, len);*/
+#if(_debug_)
+  log_dbg("tun_encaps(%s) len=%d", tun(tun,idx).devname, len);
+#endif
 
   result = tun_write(tun, pack, len, idx);
 
